-- Mock functions to use for testing HazLoot when not loaded into WoW

WoWMock = {}
WM = WoWMock

-- Corpse constructors
WM.corpse = {}

function WMCreateEmptyCorpse()
  WM:Debug("[WMCreateEmptyCorpse]")
  return {}
end
function WMCreateMoneyOnlyCorpse()
  WM:Debug("[WMCreateMoneyOnlyCorpse]")
  return {WM.CASH,}
end
function WMCreateThreeGunCorpse()
  WM:Debug("[WMCreateThreeGunCorpse]")
  return {WM.CASH,41168,41168,41168}
end
function WMCreateFiveGreens()
  WM:Debug("[WMCreateFiveGreens]")
  return {WM.CASH,42172,42172,42172,42172,42172,}
end
function WMCreateMixedItemCorpse()
  WM:Debug("[WMCreateMixedItemCorpse]")
  return {WM.CASH,41168,42172,40626,}
end

-- Constants
WM.TRASH     = 0
WM.COMMON    = 1
WM.UNCOMMON  = 2
WM.RARE      = 3
WM.EPIC      = 4
WM.LEGENDARY = 5
WM.ARTIFACT  = 6

WM.CORPSES = {"n empty"," money only"," three gun"," mixed","5 greens"}
WM.CorpseConstructors = {WMCreateEmptyCorpse,WMCreateMoneyOnlyCorpse,WMCreateThreeGunCorpse,WMCreateMixedItemCorpse,WMCreateFiveGreens}
WM.EMPTY    = 1
WM.MONEYONLY= 2
WM.THREEGUN = 3
WM.MIXED    = 4
WM.FIVEGREEN= 5

WM.PARTY    = { "Hazarakhan", "Salah", "Wexler", "Parrot", "Groupal",}
WM.GMLC     = { "Hazarakhan", "Salah", "Parrot", "Groupal",}

WM.LOOTED = -1
WM.CASH   = 0
-- itemString = "item:itemId:enchantId:jewelId1:jewelId2:jewelId3:jewelId4:suffixId:uniqueId"
WM.ITEMS = {
            [40626]={
              ["name"]="Breastplate of the Lost Protector",
              ["string"]="item:40626:0:0:0:0:0:0:0",
              ["rarity"]=WM.EPIC,
              ["itype"]="Miscellaneous", -- ??
              ["subtype"]="Other",       -- ??
              ["equiploc"]="", -- This may be wrong
              },
            [41168]={
              ["name"]="Armor Plated Combat Shotgun",
              ["string"]="item:41168:0:0:0:0:0:0:0",
              ["rarity"]=WM.EPIC,
              ["itype"]="Weapon",
              ["subtype"]="Guns",
              ["equiploc"]=18,
              },
            [42172]={
              ["name"]="Pattern: Red Lumberjack Shirt",
              ["string"]="item:42172:0:0:0:0:0:0:0",
              ["rarity"]=WM.UNCOMMON,
              ["itype"]="Recipe",
              ["subtype"]="Tailoring",
              ["equiploc"]="", -- This may be wrong
              },
            }
WM.COLOURS     = { "ffffff", "1eff00", "0080ff", "b048f8", "ff8000", "ff0000",}
WM.COLOURS[WM.TRASH] = "9d9d9d"

function WM:GetLink(id)
  local link = "|cff"
  link = link..WM.COLOURS[WM.ITEMS[id].rarity].."|H"
  link = link..WM.ITEMS[id].string.."|h["
  link = link..WM.ITEMS[id].name.."]|h|r"
  return link
end

--- Clean up the mock once you're done.
function WM:CleanUp()
  WM.corpse = {}
end

--- Instantiate a mock corpse for looting
function WM:CreateCorpse(corpsetype)
  if nil == corpsetype then corpsetype = WM.EMPTY end
  if WM.CORPSES[corpsetype] then
    WM:Debug("[CreateCorpse] Creating a"..WM.CORPSES[corpsetype].." corpse")
    WM.corpse = WM.CorpseConstructors[corpsetype]()
  else
    WM:Debug("Unimplemented corpse type")
  end
end

--- Close a corpse, re-numbering its non-empty slots, retaining the same order
function WM:CloseCorpse()
  WM:Debug("[CloseCorpse]")
  if 0 == #WM.corpse then return end
  local newCorpse = {}
  for i,item in ipairs(WM.corpse) do
    if WM.LOOTED ~= item then
      table.insert(newCorpse,item)
    end
  end
  WM.corpse = newCorpse
end

--- Create a party
function WM:AddParty()
  HazLoot:ResetMemberList(false)
  HazLoot:AddMemberToParty("Hazarakhan")
  HazLoot:AddMemberToParty("Salah")
  HazLoot:AddMemberToParty("Wexler")
  HazLoot:AddMemberToParty("Parrot")
  HazLoot:AddMemberToParty("Groupal")
end

--- LootThatBag mock functions
function GetNumLootItems()
  local tmp = nil ~= WM.corpse and #WM.corpse or 0
  WM:Debug("[Mock:GetNumLootItems] corpse has: "..tmp.." slots")
  return tmp
end

function CloseLoot()
  WM:Debug("[Mock:CloseLoot]")
end

function LootSlotIsCoin(slot)
  if not "number" == type(slot) then self:Debug("Bad arg to mock LootSlotIsCoin - not number") return nil end
  if 1 > slot or #WM.corpse < slot then self:Debug("Bad arg to mock LootSlotIsCoin - out of bounds") return nil end
  WM:Debug("[Mock:LootSlotIsCoin] "..slot..": "..(WM.CASH == WM.corpse[slot] and "yes" or "no"))
  return WM.CASH == WM.corpse[slot]
end

function LootSlot(slot)
  WM:Debug("[Mock:LootSlot]")
  if not "number" == type(slot) then self:Debug("Bad arg to mock LootSlot - not number") return nil end
  if 1 > slot or #WM.corpse < slot then self:Debug("Bad arg to mock LootSlot - out of bounds") return nil end
  WM.corpse[slot] = WM.LOOTED
  HazLoot.LootCleared()
end

function GiveMasterLoot(slot, index)
  WM:Debug("[Mock:GiveMasterLoot] item in slot "..slot.." to party member at index "..index)
  if not "number" == type(slot) then self:Debug("Bad slot to mock GiveMasterLoot - not number") return nil end
  if 1 > slot or #WM.corpse < slot then self:Debug("Bad slot to mock GiveMasterLoot - out of bounds") return nil end
  if not HazLoot.PartyMembers[index] then self:Debug("bad index to mock GiveMasterLoot - no such index") return nil end
  WM.corpse[slot] = WM.LOOTED
end

--[[ itemLink format:
    "|cff9d9d9d|Hitem:7073:0:0:0:0:0:0:0|h[Broken Fang]|h|r"
    * "|cff9d9d9d" – Colorizes
    * "|H" – "Hyperlink link data starts here"
    * "item:itemId:enchantId:jewelId1:jewelId2:jewelId3:jewelId4:suffixId:uniqueId"
    * "|h" – "End of link, text follows"
    * "[Broken Fang]" – The actual text displayed
    * "|h" – "End of hyperlink"
    * "|r" – Restores color to normal 
local found, _, itemString = string.find(itemLink, "^|c%x+|H(.+)|h%[.*%]")
]]--
function GetLootSlotLink(slot)
  if not "number" == type(slot) then self:Debug("Bad arg to mock LootSlot - not number") return nil end
  if 1 > slot or #WM.corpse < slot then self:Debug("Bad arg to mock LootSlot - out of bounds") return nil end
  local id = WM.corpse[slot]
  WM:Debug("[Mock:GetLootSlotLink] "..slot..": "..WM.ITEMS[id].name)
  return WM:GetLink(id)
end

--[[ From WoWWiki:
Returns
    "itemName", "itemLink", itemRarity, itemLevel, itemMinLevel, "itemType", "itemSubType", itemStackCount, "itemEquipLoc", "invTexture" 
    itemName 
        String - The name of the item. 
    itemLink 
        String - The item Link; you will see the clickable Link instead of the string. (String, such as: "|cFFFFFFFF|Hitem:12345:0:0:0|h[Item Name]|h|r") 
    itemRarity 
        Integer - The quality of the item. The value is 0 to 6, which represents Poor to Artifact. 
    itemLevel 
        Integer - The item level of this item. 
    itemMinLevel 
        Integer - The minimum level required to use the item, 0 meaning no level requirement. 
    itemType 
        String - The type of the item: Armor, Weapon, Quest, Key, etc. 
    itemSubType 
        String - The sub-type of the item: Enchanting, Cloth, Sword, etc. See itemType. 
    itemStackCount 
        Integer - How many of the item per stack: 20 for Runecloth, 1 for weapon, 100 for Alterac Ram Hide, etc. 
    itemEquipLoc 
        String - Where the item may be equipped, if it can. If not equippable, this is an empty string, "". The string returned is also the name of a global string variable, i.e. for "INVTYPE_HEAD", _G["INVTYPE_HEAD"] is a localized, displayable name of the location. 
    itemTexture 
        String - The name and path for the inventory icon texture for the item. 
]]--
function GetItemInfo(itemId)
  WM:Debug("[Mock:GetItemInfo]")
  return WM.ITEMS[itemId].name, WM:GetLink(itemId), WM.ITEMS[itemId].rarity, 80, 80, WM.ITEMS[itemId].itype, WM.ITEMS[itemId].subtype, 1, WM.ITEMS[itemId].equiploc, ""
end

--- Returns the name of a master loot candidate. For testing purposes, at least one
--   party member is not a candidate (out of zone, missed kill, offline, whatever)
--   This may return nil
function GetMasterLootCandidate(i)
  if not "number" == type(slot) then self:Debug("Bad arg to mock GMLC() - not number") return nil end
  if 1 > i or 40 < i then self:Debug("Bad arg to mock GMLC() - out of bounds") return nil end
  return WM.GMLC[i]
end

--- OUTPUT 
function WM:Debug(message)
  print(message)
end

